@@ -22,6 +22,7 @@ Classes:
 * :attr:`AxonaIO`
 * :attr:`AxonIO`
 * :attr:`BCI2000IO`
+* :attr:`BiocamIO`
 * :attr:`BlackrockIO`
 * :attr:`BlkIO`
 * :attr:`BrainVisionIO`
@@ -94,6 +95,10 @@ Classes:
 
     .. autoattribute:: extensions
 
+.. autoclass:: neo.io.BiocamIO
+
+    .. autoattribute:: extensions
+
 .. autoclass:: neo.io.BlackrockIO
 
     .. autoattribute:: extensions
@@ -277,6 +282,7 @@ from neo.io.asciispiketrainio import AsciiSpikeTrainIO
 from neo.io.axographio import AxographIO
 from neo.io.axonaio import AxonaIO
 from neo.io.axonio import AxonIO
+from neo.io.biocamio import BiocamIO
 from neo.io.blackrockio import BlackrockIO
 from neo.io.blkio import BlkIO
 from neo.io.bci2000io import BCI2000IO
