@@ -26,8 +26,8 @@ class NeuralynxIO(NeuralynxRawIO, BaseFromRaw):
     _prefered_signal_group_mode = 'group-by-same-units'
     mode = 'dir'
 
-    def __init__(self, dirname, use_cache=False, cache_path='same_as_resource',
-                 keep_original_times=False):
+    def __init__(self, dirname='', filename='', use_cache=False, cache_path='same_as_resource',
+                 exclude_filename=None, keep_original_times=False):
         """
         Initialise IO instance
 
@@ -41,11 +41,18 @@ class NeuralynxIO(NeuralynxRawIO, BaseFromRaw):
         cache_path : str, optional
             Folder path to use for cache files.
             Default: 'same_as_resource'
+        exclude_filename: str or list
+            Filename or list of filenames to be excluded. Expects base filenames without
+            directory path.
         keep_original_times : bool
             Preserve original time stamps as in data files. By default datasets are
             shifted to begin at t_start = 0*pq.second.
             Default: False
         """
-        NeuralynxRawIO.__init__(self, dirname=dirname, use_cache=use_cache,
-                                cache_path=cache_path, keep_original_times=keep_original_times)
-        BaseFromRaw.__init__(self, dirname)
+        NeuralynxRawIO.__init__(self, dirname=dirname, filename=filename, use_cache=use_cache,
+                                cache_path=cache_path, exclude_filename=exclude_filename,
+                                keep_original_times=keep_original_times)
+        if self.rawmode == 'one-file':
+            BaseFromRaw.__init__(self, filename)
+        elif self.rawmode == 'one-dir':
+            BaseFromRaw.__init__(self, dirname)
