@@ -304,6 +304,9 @@ class AxonaRawIO(BaseRawIO):
             i_stop = bin_dict['num_total_samples']
         if channel_indexes is None:
             channel_indexes = [i for i in range(bin_dict['num_channels'])]
+        elif isinstance(channel_indexes, slice):
+            channel_indexes_all = [i for i in range(bin_dict['num_channels'])]
+            channel_indexes = channel_indexes_all[channel_indexes]
 
         num_samples = (i_stop - i_start)
 
